<?php declare(strict_types=1);

/**
 * Handler: Handle the menus of the project.
 */

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Base\Models\Menu;
use Infomaker\Everyware\Support\Interfaces\Selectable;
use Infomaker\Everyware\Support\Collection;
use Infomaker\Everyware\Support\Str;
use WP_Term;

/**
 * Menus
 *
 * @link    http://infomaker.se
 * @package WWW\Classes\Infomaker\Everyware\Handler
 * @since   WWW\Classes\Infomaker\Everyware\Base\Menus 1.0.0
 */
class Menus implements Selectable
{

    /**
     * Contains the defaults for fetching menus via WP
     *
     * @var array
     */
    protected static $defaults = [
        'taxonomy' => 'nav_menu',
        'orderby' => 'name',
        'order' => 'ASC',
        'hide_empty' => false,
        'include' => [],
        'exclude' => [],
        'exclude_tree' => [],
        'number' => '',
        'offset' => '',
        'fields' => 'all',
        'name' => '',
        'slug' => '',
        'hierarchical' => true,
        'search' => '',
        'name__like' => '',
        'description__like' => '',
        'pad_counts' => false,
        'get' => '',
        'child_of' => 0,
        'parent' => '',
        'childless' => false,
        'cache_domain' => 'core',
        'update_term_meta_cache' => true,
        'meta_query' => ''
    ];

    /**
     * Contains the registered menu locations
     *
     * @var array
     */
    public static $locations = [];

    /**
     * Retrieves all menus
     *
     * @param array $args Optional. Array of arguments passed on to {@see get_terms()}.
     *                    Default empty array.
     *
     * @return Collection
     */
    public static function all(array $args = []): Collection
    {
        $menus = wp_get_nav_menus(array_replace_recursive(static::$defaults, $args));

        if (\is_array($menus)) {
            return new Collection(array_map(function (WP_Term $menu) {
                return new Menu($menu);
            }, $menus));
        }

        return new Collection();
    }

    /**
     * Function to retrieve all menus as select-box data
     * @return array
     */
    public static function getSelectData(): array
    {
        return static::toSelect();
    }

    /**
     * Use filter to fetch a menu
     *
     * @param array $filter 'name', 'id', 'slug' or 'location'
     *
     * @return Menu|null
     */
    public static function where($filter = []): ?Menu
    {
        if (empty($filter)) {
            return null;
        }

        $validFilters = ['name', 'id', 'slug', 'location'];

        foreach (array_keys($filter) as $filterKey) {
            if (\in_array($filterKey, $validFilters, true)) {
                return Str::is('location',
                    $filterKey) ? static::getFromLocation($filter[$filterKey]) : static::getMenu($filter[$filterKey]);
            }
        }

        return null;
    }

    /**
     * Function to fetch a menu that may be attached to a menu-location
     *
     * @param $location
     *
     * @return bool|Menu
     */
    public static function getFromLocation($location)
    {
        if (has_nav_menu($location)) {
            $themeLocations = static::getLocations();

            if (isset($themeLocations[$location])) {
                $menuObj = get_term($themeLocations[$location], 'nav_menu');

                return ! is_wp_error($menuObj) ? new Menu($menuObj) : false;
            }
        }

        return false;
    }

    /**
     * Retrieve a menu
     *
     * @param $menu
     *
     * @return bool|Menu
     */
    public static function getMenu($menu)
    {
        $menuObj = wp_get_nav_menu_object($menu);

        return ! is_wp_error($menuObj) ? new Menu($menuObj) : false;
    }

    /**
     * Add multiple menu locations at once
     *
     * @param array $locations
     *
     * @return void
     */
    public static function addLocations(array $locations): void
    {
        foreach ($locations as $location) {
            if (isset($location['id'], $location['description'])) {
                static::addLocation($location['id'], $location['description']);
            }
        }
    }

    /**
     * Add a menu location to be registered in Wordpress
     *
     * @param $id
     * @param $description
     *
     * @return void
     */
    public static function addLocation($id, $description): void
    {
        static::$locations[$id] = $description;
    }

    /**
     * Retrieve registered menu locations
     *
     * @uses  {get_nav_menu_locations()}
     * @return array
     */
    public static function getLocations(): array
    {
        return get_nav_menu_locations();
    }

    /**
     * Initiate menus
     *
     * @param null $locations
     *
     * @return void
     */
    public static function init($locations = null): void
    {

        // If there are locations to be added before registration
        if (\is_array($locations)) {
            static::addLocations($locations);
        }

        /**
         * Register the menu locations in Wordpress
         */
        add_action('init', function () {
            foreach (static::$locations as $id => $description) {
                register_nav_menu($id, $description);
            }
        });
    }

    /**
     * Return data in the form of an array fit for a select-box
     * A preselected value may be sent in.
     *
     * @param mixed $selected
     *
     * @return Collection
     */
    public static function toSelect($selected = null): Collection
    {
        return static::all()->map(function ($menu) use ($selected) {
            return [
                'value' => $menu->term_id,
                'text' => $menu->name,
                'selected' => $menu->term_id === $selected
            ];
        });
    }
}
