<?php declare(strict_types=1);

/**
 * Handler: for managing Wordpress sidebars
 */

namespace Infomaker\Everyware\Base;

/**
 * Sidebar
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Handler
 * @since   Infomaker\Everyware\Base\Sidebars 0.1
 */
class Sidebars
{

    /**
     * Contains all registered sidebars
     * @var array
     */
    protected static $sidebars = [];

    /**
     * Function to add a sidebar to the static list of sidebars
     *
     * @param string $id
     * @param array  $options
     *
     * @return void
     */
    protected static function add(string $id, array $options): void
    {
        static::$sidebars[$id] = $options;
    }

    /**
     * Function to check if a sidebar exists in the collection
     *
     * @param string $id
     *
     * @uses  is_active_sidebar()
     *
     * @return bool
     */
    public static function exist(string $id): bool
    {
        return array_key_exists($id, static::$sidebars) && is_active_sidebar($id);
    }

    /**
     * Function to get a specific sidebar
     *
     * @param string $id
     *
     * @return array|false
     */
    public static function get(string $id)
    {
        return static::exist($id) ? static::$sidebars[$id] : false;
    }

    /**
     * Function for retrieving the sidebar content set in admin
     *
     * @param string $id
     *
     * @uses  dynamic_sidebar()
     *
     * @return string
     */
    public static function getContent(string $id): string
    {
        $sidebarContent = '';

        if (static::exist($id)) {
            ob_start();
            dynamic_sidebar($id);
            $sidebarContent = ob_get_contents();
            ob_clean();
        }

        return $sidebarContent;
    }

    /**
     * Function to register our sidebars with our own defaults
     *
     * @param string $id
     * @param array  $options
     *
     * @uses  register_sidebar()
     *
     * @return void
     */
    public static function register(string $id, array $options): void
    {
        // Add new defaults
        $sidebarOptions = array_replace_recursive([
            'name' => 'No name',
            'description' => 'No description',
            'id' => $id,
            'class' => '',
            'before_widget' => '',
            'after_widget' => '',
            'before_title' => '',
            'after_title' => '',
        ], $options);

        // Add translation options
        $sidebarOptions['name'] = __($sidebarOptions['name'], $id . '-name');
        $sidebarOptions['description'] = __($sidebarOptions['description'], $id . '-description');

        static::add($id, $sidebarOptions);
        register_sidebar($sidebarOptions);
    }

    /**
     * Function for rendering the sidebar content set in admin
     *
     * @param string $id
     *
     * @uses  dynamic_sidebar()
     *
     * @return void
     */
    public static function render(string $id): void
    {
        if (static::exist($id)) {
            dynamic_sidebar($id);
        }
    }
}
